@@ -34,11 +34,13 @@
 #include <sys/types.h>
 #include <winbase.h>
 
-typedef struct {
+struct _serialport {
    HANDLE fh;
    COMMTIMEOUTS timeouts_save;
    DCB dcb_save;
-} SERIALPORT;
+};
+
+typedef struct _serialport SERIALPORT;
 
 /* Serial Port Prototypes */
 SERIALPORT *fg_serial_open ( const char *device );
