@@ -1,4 +1,4 @@
-/* $OpenBSD: pem_lib.c,v 1.48 2018/08/24 19:48:39 tb Exp $ */
+/* $OpenBSD: pem_lib.c,v 1.53 2023/07/07 13:40:44 beck Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -78,7 +78,8 @@
 #include <openssl/engine.h>
 #endif
 
-#include "asn1_locl.h"
+#include "asn1_local.h"
+#include "evp_local.h"
 
 #define MIN_LENGTH	4
 
@@ -126,6 +127,7 @@ PEM_def_callback(char *buf, int num, int w, void *key)
 	}
 	return (int)l;
 }
+LCRYPTO_ALIAS(PEM_def_callback);
 
 void
 PEM_proc_type(char *buf, int type)
@@ -145,6 +147,7 @@ PEM_proc_type(char *buf, int type)
 	strlcat(buf, str, PEM_BUFSIZE);
 	strlcat(buf, "\n", PEM_BUFSIZE);
 }
+LCRYPTO_ALIAS(PEM_proc_type);
 
 void
 PEM_dek_info(char *buf, const char *type, int len, char *str)
@@ -166,6 +169,7 @@ PEM_dek_info(char *buf, const char *type, int len, char *str)
 	buf[j + i * 2] = '\n';
 	buf[j + i * 2 + 1] = '\0';
 }
+LCRYPTO_ALIAS(PEM_dek_info);
 
 void *
 PEM_ASN1_read(d2i_of_void *d2i, const char *name, FILE *fp, void **x,
@@ -183,6 +187,7 @@ PEM_ASN1_read(d2i_of_void *d2i, const char *name, FILE *fp, void **x,
 	BIO_free(b);
 	return (ret);
 }
+LCRYPTO_ALIAS(PEM_ASN1_read);
 
 static int
 check_pem(const char *nm, const char *name)
@@ -263,6 +268,16 @@ check_pem(const char *nm, const char *name)
 	    !strcmp(name, PEM_STRING_PKCS7))
 		return 1;
 
+#ifndef OPENSSL_NO_CMS
+	if (strcmp(nm, PEM_STRING_X509) == 0 &&
+	    strcmp(name, PEM_STRING_CMS) == 0)
+		return 1;
+
+	/* Allow CMS to be read from PKCS#7 headers */
+	if (strcmp(nm, PEM_STRING_PKCS7) == 0 &&
+	    strcmp(name, PEM_STRING_CMS) == 0)
+		return 1;
+#endif
 
 	return 0;
 }
@@ -311,6 +326,7 @@ err:
 		free(data);
 	return ret;
 }
+LCRYPTO_ALIAS(PEM_bytes_read_bio);
 
 int
 PEM_ASN1_write(i2d_of_void *i2d, const char *name, FILE *fp, void *x,
@@ -329,6 +345,7 @@ PEM_ASN1_write(i2d_of_void *i2d, const char *name, FILE *fp, void *x,
 	BIO_free(b);
 	return (ret);
 }
+LCRYPTO_ALIAS(PEM_ASN1_write);
 
 int
 PEM_ASN1_write_bio(i2d_of_void *i2d, const char *name, BIO *bp, void *x,
@@ -427,6 +444,7 @@ err:
 	freezero(data, (unsigned int)dsize);
 	return (ret);
 }
+LCRYPTO_ALIAS(PEM_ASN1_write_bio);
 
 int
 PEM_do_header(EVP_CIPHER_INFO *cipher, unsigned char *data, long *plen,
@@ -472,6 +490,7 @@ PEM_do_header(EVP_CIPHER_INFO *cipher, unsigned char *data, long *plen,
 	*plen = j + i;
 	return (1);
 }
+LCRYPTO_ALIAS(PEM_do_header);
 
 int
 PEM_get_EVP_CIPHER_INFO(char *header, EVP_CIPHER_INFO *cipher)
@@ -533,6 +552,7 @@ PEM_get_EVP_CIPHER_INFO(char *header, EVP_CIPHER_INFO *cipher)
 
 	return (1);
 }
+LCRYPTO_ALIAS(PEM_get_EVP_CIPHER_INFO);
 
 static int
 load_iv(char **fromp, unsigned char *to, int num)
@@ -579,6 +599,7 @@ PEM_write(FILE *fp, const char *name, const char *header,
 	BIO_free(b);
 	return (ret);
 }
+LCRYPTO_ALIAS(PEM_write);
 
 int
 PEM_write_bio(BIO *bp, const char *name, const char *header,
@@ -597,8 +618,7 @@ PEM_write_bio(BIO *bp, const char *name, const char *header,
 	    (BIO_write(bp, "-----\n", 6) != 6))
 		goto err;
 
-	i = strlen(header);
-	if (i > 0) {
+	if (header != NULL && (i = strlen(header)) > 0) {
 		if ((BIO_write(bp, header, i) != i) ||
 		    (BIO_write(bp, "\n", 1) != 1))
 			goto err;
@@ -637,6 +657,7 @@ err:
 	PEMerror(reason);
 	return (0);
 }
+LCRYPTO_ALIAS(PEM_write_bio);
 
 int
 PEM_read(FILE *fp, char **name, char **header, unsigned char **data, long *len)
@@ -653,6 +674,7 @@ PEM_read(FILE *fp, char **name, char **header, unsigned char **data, long *len)
 	BIO_free(b);
 	return (ret);
 }
+LCRYPTO_ALIAS(PEM_read);
 
 int
 PEM_read_bio(BIO *bp, char **name, char **header, unsigned char **data,
@@ -825,6 +847,7 @@ err:
 	BUF_MEM_free(dataB);
 	return (0);
 }
+LCRYPTO_ALIAS(PEM_read_bio);
 
 /* Check pem string and return prefix length.
  * If for example the pem_str == "RSA PRIVATE KEY" and suffix = "PRIVATE KEY"
