@@ -17,6 +17,7 @@
 
 #include <log4cxx/logstring.h>
 #include <log4cxx/helpers/writer.h>
+#include <log4cxx/helpers/loglog.h>
 #include <stdexcept>
 
 using namespace log4cxx::helpers;
@@ -30,10 +31,3 @@ Writer::Writer()
 Writer::~Writer()
 {
 }
-
-#ifdef LOG4CXX_MULTI_PROCESS
-OutputStreamPtr Writer::getOutPutStreamPtr()
-{
-	throw std::logic_error("getOutPutStreamPtr must be implemented in the derived class that you are using");
-}
-#endif
