@@ -18,11 +18,6 @@
 #if !defined(_LOG4CXX_ROLLING_GZ_COMPRESS_ACTION_H)
 #define _LOG4CXX_ROLLING_GZ_COMPRESS_ACTION_H
 
-#if defined(_MSC_VER)
-	#pragma warning ( push )
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
 #include <log4cxx/rolling/action.h>
 #include <log4cxx/file.h>
 
@@ -34,9 +29,7 @@ namespace rolling
 
 class GZCompressAction : public Action
 {
-		const File source;
-		const File destination;
-		bool deleteSource;
+		struct GZCompressActionPrivate;
 	public:
 		DECLARE_ABSTRACT_LOG4CXX_OBJECT(GZCompressAction)
 		BEGIN_LOG4CXX_CAST_MAP()
@@ -50,13 +43,25 @@ class GZCompressAction : public Action
 		GZCompressAction(const File& source,
 			const File& destination,
 			bool deleteSource);
+		~GZCompressAction();
 
 		/**
 		 * Perform action.
 		 *
 		 * @return true if successful.
 		 */
-		virtual bool execute(log4cxx::helpers::Pool& pool) const;
+		bool execute(log4cxx::helpers::Pool& pool) const override;
+
+		/**
+		 * Set to true to throw an IOException on a fork failure.  By default, this
+		 * is true.  When an IOException is thrown, this will automatically cause the
+		 * error handler to be called(which is the recommended way of handling this
+		 * problem).  By setting this to false, the GZCompressAction effectively
+		 * turns into a FileRenameAction if any errors are encountered.
+		 *
+		 * @param throwIO
+		 */
+		void setThrowIOExceptionOnForkFailure(bool throwIO);
 
 	private:
 		GZCompressAction(const GZCompressAction&);
@@ -68,9 +73,5 @@ LOG4CXX_PTR_DEF(GZCompressAction);
 }
 }
 
-#if defined(_MSC_VER)
-	#pragma warning ( pop )
-#endif
-
 #endif
 
