@@ -1,6 +1,6 @@
 /* winMemoryMappedFile.cpp: memory mapped files
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -24,6 +24,7 @@
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 
+#include <miktex/Core/BufferSizes>
 #include <miktex/Trace/Trace>
 #include <miktex/Util/PathNameUtil>
 
@@ -32,10 +33,11 @@
 #include "winMemoryMappedFile.h"
 #include "Utils/inliners.h"
 
+using namespace std;
+
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Trace;
 using namespace MiKTeX::Util;
-using namespace std;
 
 MemoryMappedFile* MemoryMappedFile::Create()
 {
@@ -65,15 +67,16 @@ void* winMemoryMappedFile::Open(const PathName& path_, bool readWrite)
   this->readWrite = readWrite;
 
   // create a unique object name
+  PathName path2 = path;
+  path2.TransformForComparison();
   name = "";
-  name.reserve(BufferSizes::MaxPath);
-  for (size_t i = 0; path[i] != 0; ++i)
+  for (size_t i = 0; path2[i] != 0; ++i)
   {
-    if (PathNameUtil::IsDirectoryDelimiter(path[i]) || path[i] == ':')
+    if (PathNameUtil::IsDirectoryDelimiter(path2[i]) || path2[i] == ':')
     {
       continue;
     }
-    name += ToLower(path[i]);
+    name +=path2[i];
   }
 
   // try to open an existing file mapping
@@ -110,7 +113,6 @@ void* winMemoryMappedFile::Open(const PathName& path_, bool readWrite)
   else
   {
     DWORD lastError = GetLastError();
-    MIKTEX_ASSERT(lastError != ERROR_SUCCESS);
     if (lastError != ERROR_FILE_NOT_FOUND)
     {
       MIKTEX_FATAL_WINDOWS_RESULT_2("OpenFileMappingW", lastError, "path", name);
