@@ -1,6 +1,6 @@
-/* 3-1.cpp:
+/* 5-1.cpp:
 
-   Copyright (C) 1996-2017 Christian Schenk
+   Copyright (C) 2018-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -23,18 +23,17 @@
 
 #include <miktex/Core/Test>
 
+using namespace MiKTeX::Core;
 using namespace MiKTeX::Test;
+using namespace MiKTeX::Util;
 
-BEGIN_TEST_SCRIPT("process-3-1");
+BEGIN_TEST_SCRIPT("process-5-1");
 
 BEGIN_TEST_FUNCTION(1);
 {
-  char buf[100];
-  size_t n;
-  while ((n = fread(buf, 1, 100, stdin)) > 0)
-  {
-    fwrite(buf, 1, n, stdout);
-  }
+  PathName file(vecArgs[0]);
+  file.MakeFullyQualified();
+  pSession->ScheduleSystemCommand("echo hello>" + file.ToString());
 }
 END_TEST_FUNCTION();
 
