@@ -2,7 +2,7 @@
 ** BasicDVIReader.cpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2024 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -84,7 +84,7 @@ int BasicDVIReader::evalCommand (CommandHandler &handler, int &param) {
 
 	const int opcode = readByte();
 	if (!isStreamValid() || opcode < 0)  // at end of file
-		throw InvalidDVIFileException("invalid DVI file");
+		throw DVIPrematureEOFException();
 
 	int num_param_bytes = 0;
 	param = -1;
@@ -166,7 +166,7 @@ void BasicDVIReader::executePreamble () {
 			return;
 		}
 	}
-	throw DVIException("invalid DVI file");
+	throw DVIException("invalid DVI file (missing preamble)");
 }
 
 
@@ -174,7 +174,7 @@ void BasicDVIReader::executePreamble () {
 void BasicDVIReader::goToPostamble () {
 	clearStream();
 	if (!isStreamValid())
-		throw DVIException("invalid DVI file");
+		throw DVIException("invalid DVI file (missing postamble)");
 
 	seek(-1, ios::end);  // stream pointer to last byte
 	int count=0;
@@ -201,7 +201,7 @@ void BasicDVIReader::executePostamble () {
 void BasicDVIReader::executePostPost () {
 	clearStream();  // reset all status bits
 	if (!isStreamValid())
-		throw DVIException("invalid DVI file");
+		throw DVIException("invalid DVI file (missing postpost)");
 
 	seek(-1, ios::end);       // stream pointer to last byte
 	int count=0;
