@@ -2,7 +2,7 @@
 ** Message.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2024 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -192,6 +192,15 @@ MessageStream& Message::estream (bool prefix) {
 }
 
 
+/** Returns the output stream for user messages
+ *  @param[in] always ignore verbosity settings if true */
+MessageStream& Message::ustream (bool always) {
+	init();
+	MessageStream *ms = (always || (LEVEL & USERMESSAGES)) ? &messageStream : &nullStream;
+	return *ms;
+}
+
+
 static bool colorchar2int (char colorchar, int *val) {
 	colorchar = tolower(colorchar);
 	if (colorchar >= '0' && colorchar <= '9')
