@@ -2,7 +2,7 @@
 ** PSInterpreter.cpp                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2024 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -30,7 +30,6 @@
 #include "FileSystem.hpp"
 #include "InputReader.hpp"
 #include "Message.hpp"
-#include "PSFilter.hpp"
 #include "PSInterpreter.hpp"
 #include "SignalHandler.hpp"
 #include "utility.hpp"
@@ -85,16 +84,6 @@ void PSInterpreter::init () {
 }
 
 
-/** Sets or replaces the filter applied to the PS code.
- *  @param[in] filter the new filter being used
- *  @return the previous, replaced filter (nullptr if there was none) */
-PSFilter* PSInterpreter::setFilter (PSFilter *filter) {
-	PSFilter *prevFilter = _filter;
-	_filter = filter;
-	return prevFilter;
-}
-
-
 PSActions* PSInterpreter::setActions (PSActions *actions) {
 	PSActions *old_actions = _actions;
 	_actions = actions;
@@ -140,15 +129,6 @@ bool PSInterpreter::execute (const char *str, size_t len, bool flush) {
 		complete = true;
 	}
 
-	if (_filter && _filter->active()) {
-		PSFilter *filter = _filter;
-		_filter = nullptr;       // prevent recursion when filter calls execute()
-		filter->execute(str, len);
-		if (filter->active())    // filter still active after execution?
-			_filter = filter;
-		return complete;
-	}
-
 	// feed Ghostscript with code chunks that are not larger than 64KB
 	// => see documentation of gsapi_run_string_foo()
 	const char *p=str;
